<?php

namespace Drupal\cron_fail_alert\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\State\StateInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Cron Fail Alert event subscriber.
 */
class CronFailAlertSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * The cron fail alert settings object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $cronFailAlertSettings;

  /**
   * The site config object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $systemSite;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected LoggerChannelFactoryInterface $loggerFactory;

  /**
   * The mail manager service.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected MailManagerInterface $mailManager;

  /**
   * The state key/value store.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected StateInterface $state;

  /**
   * Constructs event subscriber.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state key/value store.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LanguageManagerInterface $language_manager, LoggerChannelFactoryInterface $logger_factory, MailManagerInterface $mail_manager, StateInterface $state) {
    $this->cronFailAlertSettings = $config_factory->get('cron_fail_alert.settings');
    $this->systemSite = $config_factory->get('system.site');
    $this->languageManager = $language_manager;
    $this->loggerFactory = $logger_factory;
    $this->mailManager = $mail_manager;
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::RESPONSE => ['onKernelResponse'],
    ];
  }

  /**
   * Runs check on the cron job status and sends email alerts if it has failed.
   */
  public function onKernelResponse(): void {
    // Defines how often we want to run the checks (in minutes).
    $frequency = $this->cronFailAlertSettings->get('frequency');

    // Only proceed if it's been longer than given frequency minutes since the
    // last check.
    $last_check = $this->state->get('cron_fail_alert_last_check_timestamp', 0);
    if ((time() - $last_check) / 60 < $frequency) {
      return;
    }

    // More than given frequency min have passed since the last check. So, let's
    // proceed with a new check.
    if ($this->cronStatusCheck()) {
      // Check was successful. Now update the 'last check timestamp' variable so
      // the next check runs once frequency min lapsed from now.
      $this->state->set('cron_fail_alert_last_check_timestamp', time());
    }
  }

  /**
   * Compares last cron run time with a set tolerance to check for any failures.
   *
   * @return bool
   *   Returns TRUE if the cron has failed, FALSE otherwise.
   */
  protected function cronStatusCheck(): bool {
    // Calculate the time since the last time cron run.
    $lastCron = $this->state->get('system.cron_last');
    $minutesAgo = (time() - $lastCron) / 60;

    // Check if cron has failed by comparing the elapsed time with the set
    // tolerance.
    $tolerance = $this->cronFailAlertSettings->get('tolerance');
    if (!empty($tolerance) && $minutesAgo >= $tolerance) {
      // If cron has failed, send an email alert.
      $to = $this->cronFailAlertSettings->get('to') ?? $this->systemSite->get('mail');
      $subject = $this->cronFailAlertSettings->get('subject') . ' on Site: ' . $this->systemSite->get('name');
      $message = $this->t($this->cronFailAlertSettings->get('message'), [
        '@minutes' => floor($minutesAgo),
      ]);
      $mailSent = $this->cronFailAlertSendEmail($to, $subject, $message);
      if ($mailSent) {
        return TRUE;
      }
    }

    // Return FALSE if cron has not failed.
    return FALSE;
  }

  /**
   * Sends an email notification to the specified recipient.
   *
   * @param string $to
   *   The email address of the recipient.
   * @param string $subject
   *   The subject of the email.
   * @param string $body
   *   The body of the email.
   *
   * @return bool
   *   TRUE if the email was sent successfully, otherwise FALSE.
   */
  protected function cronFailAlertSendEmail(string $to, string $subject, string $body): bool {
    // Set the module and key values to identify the email template to use.
    $module = 'cron_fail_alert';
    $key = 'cron_fail_mail';

    // Set the email parameters to be used in the email template.
    $params['title'] = $subject;
    $params['message'] = $body;

    // Get the default language code.
    $langcode = $this->languageManager->getDefaultLanguage()->getId();

    // Build the email and attempt to send it.
    $result = $this->mailManager->mail($module, $key, $to, $langcode, $params);

    // Check if the email was sent successfully.
    if (!$result['result']) {
      // If sending the email failed, log an error message.
      $message = $this->t('There was a problem sending your email notification to @email.', ['@email' => $to]);
      $this->loggerFactory->get('cron_fail_alert')->error($message);

      // Return FALSE to indicate that the email was not sent successfully.
      return FALSE;
    }

    // If the email was sent successfully, log a success message.
    $message = $this->t('An email notification has been sent to @email', ['@email' => $to]);
    $this->loggerFactory->get('cron_fail_alert')->notice($message);

    // Return TRUE to indicate that the email was sent successfully.
    return TRUE;
  }

}
